% 2-D radially symmetric beam propagator, DHT based method

% parameters - should be same as in p1FFT.m
lambda = 0.633e-06;          
LR     = 1.5e-02;
NR     = 2048;
dz     = 0.05;
LZ     = 1.0;

% derived parameters
k0 = 2*pi/lambda;
stps = LZ/dz;

% prepare Hankel transofrm
tstart = cputime;
HT = myDHT(LR,NR);
fprintf(1,'CPU time in initialization: %g\n',cputime - tstart);


% amplitude holder, define an initial condition
am = zeros(1,NR);
for x=1:NR
    am(x) = IC(sqrt(HT.cr(x)^2));
end
am = am';

% propagator
pr = zeros(1,NR);
for x=1:NR
   pr(x) = exp(-i*(HT.kt(x)^2)/(2*k0)*dz );
%    pr(x) = exp(i*dz*(sqrt(k0*k0 - HT.kt(x)^2) - k0) );
end
pr = pr';

% absorbing boundary guard
br = zeros(1,NR);
for x=1:NR
    br(x) = exp(-( (HT.cr(x)^2)/(LR*LR/1.25)).^8 );
end
br = br';

% define one linear step
LinearStep = @(amplitudein,propagator) HT.T*( propagator.*(HT.T*amplitudein) );


% evolve amplitude
for s=1:stps
  fprintf(1,'executing %d out of %d, distance = %d\n',s,stps,s*dz);
  am = LinearStep(am,pr);
  am = am.*br;
end

% symmetrize radial functions for better viewwing
am2save = abs([flipud(am);am]);
rc2save = [flipud(-HT.cr);HT.cr];

plot(abs([flipud(am);am]).^2);

% save for comparison with p1FFT.m result
fout = fopen('amplitude_vs_radius_DHT.dat','w');
for x=1:2*NR
    fprintf(fout,'%g %g\n',rc2save(x),am2save(x));
end


